@@ -5,7 +5,7 @@ import numpy as np
 from numpy.typing import NDArray
 
 from pyobs.interfaces import IAutoFocus
-from pyobs.events import FocusFoundEvent
+from pyobs.events import FocusFoundEvent, BadWeatherEvent, Event
 from pyobs.interfaces import IExposureTime, IImageType, IFocuser, IFilters, IData
 from pyobs.object import get_object
 from pyobs.mixins import CameraSettingsMixin
@@ -31,6 +31,8 @@ class AutoFocusSeries(Module, CameraSettingsMixin, IAutoFocus):
         filters: Optional[Union[str, IFilters]] = None,
         filter_name: Optional[str] = None,
         binning: Optional[int] = None,
+        broadcast: bool = False,
+        final_image: bool = True,
         **kwargs: Any,
     ):
         """Initialize a new auto focus system.
@@ -41,6 +43,8 @@ class AutoFocusSeries(Module, CameraSettingsMixin, IAutoFocus):
             filters: Name of IFilters, if any.
             filter_name: Name of filter to set.
             offset: If True, offsets are used instead of absolute focus values.
+            broadcast: Whether to broadcast focus series images.
+            final_image: Whether to take final image with optimal focus, which is always broadcasted.
         """
         Module.__init__(self, **kwargs)
 
@@ -50,6 +54,9 @@ class AutoFocusSeries(Module, CameraSettingsMixin, IAutoFocus):
         self._filters = filters
         self._offset = offset
         self._abort = threading.Event()
+        self._running = False
+        self._broadcast = broadcast
+        self._final_image = final_image
 
         # create focus series
         self._series: FocusSeries = get_object(series, FocusSeries)
@@ -73,6 +80,7 @@ class AutoFocusSeries(Module, CameraSettingsMixin, IAutoFocus):
 
         # register event
         await self.comm.register_event(FocusFoundEvent)
+        await self.comm.register_event(BadWeatherEvent, self._on_bad_weather)
 
         # check focuser and camera
         try:
@@ -100,8 +108,16 @@ class AutoFocusSeries(Module, CameraSettingsMixin, IAutoFocus):
         Raises:
             FileNotFoundException: If image could not be downloaded.
         """
-        log.info("Performing auto-focus...")
 
+        try:
+            log.info("Performing auto-focus...")
+            self._running = True
+            return await self._auto_focus(count, step, exposure_time, **kwargs)
+
+        finally:
+            self._running = False
+
+    async def _auto_focus(self, count: int, step: float, exposure_time: float, **kwargs: Any) -> Tuple[float, float]:
         # get focuser
         log.info("Getting proxy for focuser...")
         focuser = await self.proxy(self._focuser, IFocuser)
@@ -112,6 +128,8 @@ class AutoFocusSeries(Module, CameraSettingsMixin, IAutoFocus):
 
         # do camera settings
         await self._do_camera_settings(camera)
+        if isinstance(camera, IImageType):
+            await camera.set_image_type(ImageType.FOCUS)
 
         # get filter wheel and current filter
         filter_name = "unknown"
@@ -163,11 +181,7 @@ class AutoFocusSeries(Module, CameraSettingsMixin, IAutoFocus):
             if self._abort.is_set():
                 raise exceptions.AbortedError()
             try:
-                if isinstance(camera, IExposureTime):
-                    await camera.set_exposure_time(exposure_time)
-                if isinstance(camera, IImageType):
-                    await camera.set_image_type(ImageType.FOCUS)
-                filename = await camera.grab_data()
+                filename = await self._take_image(camera, exposure_time)
             except exc.RemoteError:
                 log.error("Could not take image.")
                 continue
@@ -181,14 +195,17 @@ class AutoFocusSeries(Module, CameraSettingsMixin, IAutoFocus):
             try:
                 self._series.analyse_image(image, foc)
             except:
-                # do nothing..
+                # do nothing...
                 log.error("Could not analyse image.")
                 continue
 
         # fit focus
         if self._abort.is_set():
             raise exceptions.AbortedError()
-        focus = self._series.fit_focus()
+        try:
+            focus = self._series.fit_focus()
+        except Exception as e:
+            raise exc.GeneralError(f"Could not calculate best focus: {e}")
 
         # did focus series fail?
         if focus is None or focus[0] is None or np.isnan(focus[0]):
@@ -218,9 +235,21 @@ class AutoFocusSeries(Module, CameraSettingsMixin, IAutoFocus):
         # send event
         await self.comm.send_event(FocusFoundEvent(absolute, focus[1], filter_name))
 
+        # take final image?
+        if self._final_image:
+            await self._take_image(camera, exposure_time)
+
         # return result
         return focus[0], focus[1]
 
+    async def _take_image(self, camera, exposure_time):
+        if isinstance(camera, IExposureTime):
+            await camera.set_exposure_time(exposure_time)
+        if isinstance(camera, IData):
+            return await camera.grab_data(broadcast=self._broadcast)
+        else:
+            raise exc.GeneralError("Cannot grab data from camera.")
+
     async def auto_focus_status(self, **kwargs: Any) -> Dict[str, Any]:
         """Returns current status of auto focus.
 
@@ -236,5 +265,24 @@ class AutoFocusSeries(Module, CameraSettingsMixin, IAutoFocus):
         """Abort current actions."""
         self._abort.set()
 
+    async def _on_bad_weather(self, event: Event, sender: str) -> bool:
+        """Abort series if a bad weather event occurs.
+
+        Args:
+            event: The bad weather event.
+            sender: Who sent it.
+        """
+
+        # check
+        if not isinstance(event, BadWeatherEvent):
+            raise ValueError("Wrong event type.")
+
+        # park
+        if self._running:
+            log.warning("Aborting focus series due to bad weather.")
+            self._abort.set()
+            return True
+        return False
+
 
 __all__ = ["AutoFocusSeries"]
